@@ -7,100 +7,99 @@ import sys
 import time
 import pytest
 import socket
-from libensemble.resources.resources import ResourcesException
+from libensemble.resources.mpi_resources import MPIResourcesException
 from libensemble.executors.executor import Executor, ExecutorException, TimeoutExpired
 from libensemble.executors.executor import NOT_STARTED_STATES
 
 
 USE_BALSAM = False
-
 NCORES = 1
-sim_app = './my_simtask.x'
+build_sims = ['my_simtask.c', 'my_serialtask.c', 'c_startup.c']
+
+sim_app = 'simdir/my_simtask.x'
+serial_app = 'simdir/my_serialtask.x'
+c_startup = 'simdir/c_startup.x'
+py_startup = 'simdir/py_startup.py'
 
 
 def setup_module(module):
-    print("setup_module      module:%s" % module.__name__)
+    try:
+        print("setup_module module:%s" % module.__name__)
+    except AttributeError:
+        print("setup_module (direct run) module:%s" % module)
     if Executor.executor is not None:
         del Executor.executor
         Executor.executor = None
+    build_simfuncs()
 
 
 def setup_function(function):
-    print("setup_function    function:%s" % function.__name__)
+    print("setup_function function:%s" % function.__name__)
     if Executor.executor is not None:
         del Executor.executor
         Executor.executor = None
 
 
 def teardown_module(module):
-    print("teardown_module   module:%s" % module.__name__)
+    try:
+        print("teardown_module module:%s" % module.__name__)
+    except AttributeError:
+        print("teardown_module (direct run) module:%s" % module)
     if Executor.executor is not None:
         del Executor.executor
         Executor.executor = None
 
 
-def build_simfunc():
+def build_simfuncs():
     import subprocess
-
-    # Build simfunc
-    # buildstring='mpif90 -o my_simtask.x my_simtask.f90' # On cray need to use ftn
-    buildstring = 'mpicc -o my_simtask.x simdir/my_simtask.c'
-    # subprocess.run(buildstring.split(), check=True) # Python3.5+
-    subprocess.check_call(buildstring.split())
+    for sim in build_sims:
+        app_name = '.'.join([sim.split('.')[0], 'x'])
+        if not os.path.isfile(app_name):
+            buildstring = 'mpicc -o ' + os.path.join('simdir', app_name) + ' ' + os.path.join('simdir', sim)
+            subprocess.check_call(buildstring.split())
 
 
 # This would typically be in the user calling script
-# Cannot test auto_resources here - as no workers set up.
 def setup_executor():
-    # sim_app = './my_simtask.x'
-    if not os.path.isfile(sim_app):
-        build_simfunc()
-
+    """Set up an MPI Executor with sim app"""
     if USE_BALSAM:
         from libensemble.executors.balsam_executor import BalsamMPIExecutor
-        exctr = BalsamMPIExecutor(auto_resources=False)
+        exctr = BalsamMPIExecutor()
     else:
         from libensemble.executors.mpi_executor import MPIExecutor
-        exctr = MPIExecutor(auto_resources=False)
-
-    exctr.register_calc(full_path=sim_app, calc_type='sim')
+        exctr = MPIExecutor()
+    exctr.register_app(full_path=sim_app, calc_type='sim')
 
 
-def setup_executor_noreg():
-    # sim_app = './my_simtask.x'
-    if not os.path.isfile(sim_app):
-        build_simfunc()
+def setup_serial_executor():
+    """Set up serial Executor"""
+    from libensemble.executors.executor import Executor
+    exctr = Executor()
+    exctr.register_app(full_path=serial_app, calc_type='sim')
 
-    if USE_BALSAM:
-        from libensemble.executors.balsam_executor import BalsamMPIExecutor
-        exctr = BalsamMPIExecutor(auto_resources=False)
-    else:
-        from libensemble.executors.mpi_executor import MPIExecutor
-        exctr = MPIExecutor(auto_resources=False)
 
-    exctr.register_calc(full_path=sim_app, calc_type='sim')
+def setup_executor_startups():
+    """Set up serial Executor"""
+    from libensemble.executors.executor import Executor
+    exctr = Executor()
+    exctr.register_app(full_path=c_startup, app_name='c_startup')
+    exctr.register_app(full_path=py_startup, app_name='py_startup')
 
 
 def setup_executor_noapp():
-    # sim_app = './my_simtask.x'
-    if not os.path.isfile(sim_app):
-        build_simfunc()
-
+    """Set up an MPI Executor but do not register application"""
     if USE_BALSAM:
         from libensemble.executors.balsam_executor import BalsamMPIExecutor
-        exctr = BalsamMPIExecutor(auto_resources=False)
+        exctr = BalsamMPIExecutor()
     else:
         from libensemble.executors.mpi_executor import MPIExecutor
-        exctr = MPIExecutor(auto_resources=False)
+        exctr = MPIExecutor()
         if exctr.workerID is not None:
             sys.exit("Something went wrong in creating Executor")
 
 
 def setup_executor_fakerunner():
-    # sim_app = './my_simtask.x'
-    if not os.path.isfile(sim_app):
-        build_simfunc()
-
+    """Set up an MPI Executor with a non-existent MPI runner"""
     if USE_BALSAM:
         print('Balsom does not support this feature - running MPIExecutor')
 
@@ -110,14 +109,14 @@ def setup_executor_fakerunner():
                   'subgroup_launch': True}
 
     from libensemble.executors.mpi_executor import MPIExecutor
-    exctr = MPIExecutor(auto_resources=False, custom_info=customizer)
-    exctr.register_calc(full_path=sim_app, calc_type='sim')
+    exctr = MPIExecutor(custom_info=customizer)
+    exctr.register_app(full_path=sim_app, calc_type='sim')
 
 
 # -----------------------------------------------------------------------------
 # The following would typically be in the user sim_func
-def polling_loop(exctr, task, timeout_sec=0.5, delay=0.05):
-    # import time
+def polling_loop(exctr, task, timeout_sec=1, delay=0.05):
+    """Iterate over a loop, polling for an exit condition"""
     start = time.time()
 
     while time.time() - start < timeout_sec:
@@ -126,7 +125,7 @@ def polling_loop(exctr, task, timeout_sec=0.5, delay=0.05):
         # Check output file for error
         if task.stdout_exists():
             if 'Error' in task.read_stdout():
-                print("Found(deliberate) Error in ouput file - cancelling task")
+                print("Found(deliberate) Error in output file - cancelling task")
                 exctr.kill(task)
                 time.sleep(delay)  # Give time for kill
                 break
@@ -149,7 +148,7 @@ def polling_loop(exctr, task, timeout_sec=0.5, delay=0.05):
 
 
 def polling_loop_multitask(exctr, task_list, timeout_sec=4.0, delay=0.05):
-    # import time
+    """Iterate over a loop, polling for exit conditions on multiple tasks"""
     start = time.time()
 
     while time.time() - start < timeout_sec:
@@ -174,7 +173,7 @@ def polling_loop_multitask(exctr, task_list, timeout_sec=4.0, delay=0.05):
                 # Check output file for error
                 if task.stdout_exists():
                     if 'Error' in task.read_stdout():
-                        print("Found (deliberate) Error in ouput file - cancelling task %d" % (task.id))
+                        print("Found (deliberate) Error in output file - cancelling task %d" % (task.id))
                         exctr.kill(task)
                         time.sleep(delay)  # Give time for kill
                         continue
@@ -229,14 +228,14 @@ def test_launch_and_wait_timeout():
     assert task.state == 'USER_KILLED', "task.state should be USER_KILLED. Returned " + str(task.state)
 
 
-def test_launch_wait_on_run():
-    """ Test of launching task with wait_on_run """
+def test_launch_wait_on_start():
+    """ Test of launching task with wait_on_start """
     print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
     setup_executor()
     exctr = Executor.executor
     cores = NCORES
     args_for_sim = 'sleep 0.2'
-    task = exctr.submit(calc_type='sim', num_procs=cores, app_args=args_for_sim, wait_on_run=True)
+    task = exctr.submit(calc_type='sim', num_procs=cores, app_args=args_for_sim, wait_on_start=True)
     assert task.state not in NOT_STARTED_STATES, "Task should not be in a NOT_STARTED state. State: " + str(task.state)
     exctr.poll(task)
     if not task.finished:
@@ -270,6 +269,18 @@ def test_kill_on_timeout():
     assert task.state == 'USER_KILLED', "task.state should be USER_KILLED. Returned " + str(task.state)
 
 
+def test_kill_on_timeout_polling_loop_method():
+    print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
+    setup_executor()
+    exctr = Executor.executor
+    cores = NCORES
+    args_for_sim = 'sleep 10'
+    task = exctr.submit(calc_type='sim', num_procs=cores, app_args=args_for_sim)
+    exctr.polling_loop(task, timeout=1)
+    assert task.finished, "task.finished should be True. Returned " + str(task.finished)
+    assert task.state == 'USER_KILLED', "task.state should be USER_KILLED. Returned " + str(task.state)
+
+
 def test_launch_and_poll_multitasks():
     print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
     setup_executor()
@@ -319,7 +330,7 @@ def test_get_task():
 
 @pytest.mark.timeout(30)
 def test_procs_and_machinefile_logic():
-    """ Test of supplying various input configurations when auto_resources is False."""
+    """ Test of supplying various input configurations."""
     print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
 
     # Note: Could test task_partition routine directly - without launching tasks...
@@ -336,50 +347,50 @@ def test_procs_and_machinefile_logic():
             f.write(socket.gethostname() + '\n')
 
     task = exctr.submit(calc_type='sim', machinefile=machinefilename, app_args=args_for_sim)
-    task = polling_loop(exctr, task, delay=0.02)
+    task = polling_loop(exctr, task, delay=0.05)
     assert task.finished, "task.finished should be True. Returned " + str(task.finished)
     assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
 
-    # Testing num_procs = num_nodes*ranks_per_node (shouldn't fail)
-    task = exctr.submit(calc_type='sim', num_procs=6, num_nodes=2, ranks_per_node=3, app_args=args_for_sim)
-    task = polling_loop(exctr, task, delay=0.02)
+    # Testing num_procs = num_nodes*procs_per_node (shouldn't fail)
+    task = exctr.submit(calc_type='sim', num_procs=6, num_nodes=2, procs_per_node=3, app_args=args_for_sim)
+    task = polling_loop(exctr, task, delay=0.05)
     assert task.finished, "task.finished should be True. Returned " + str(task.finished)
     assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
 
-    # Testing num_procs not num_nodes*ranks_per_node (should fail)
+    # Testing num_procs not num_nodes*procs_per_node (should fail)
     try:
-        task = exctr.submit(calc_type='sim', num_procs=9, num_nodes=2, ranks_per_node=5, app_args=args_for_sim)
-    except ResourcesException as e:
-        assert e.args[0] == 'num_procs does not equal num_nodes*ranks_per_node'
+        task = exctr.submit(calc_type='sim', num_procs=9, num_nodes=2, procs_per_node=5, app_args=args_for_sim)
+    except MPIResourcesException as e:
+        assert e.args[0] == 'num_procs does not equal num_nodes*procs_per_node'
     else:
         assert 0
 
     # Testing no num_procs (shouldn't fail)
-    task = exctr.submit(calc_type='sim', num_nodes=2, ranks_per_node=3, app_args=args_for_sim)
+    task = exctr.submit(calc_type='sim', num_nodes=2, procs_per_node=3, app_args=args_for_sim)
     assert 1
-    task = polling_loop(exctr, task, delay=0.02)
+    task = polling_loop(exctr, task, delay=0.05)
     assert task.finished, "task.finished should be True. Returned " + str(task.finished)
     assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
 
     # Testing nothing given (should fail)
     try:
         task = exctr.submit(calc_type='sim', app_args=args_for_sim)
-    except ResourcesException as e:
-        assert e.args[0] == 'Need num_procs, num_nodes/ranks_per_node, or machinefile'
+    except MPIResourcesException as e:
+        assert e.args[0] == 'Need num_procs, num_nodes/procs_per_node, or machinefile'
     else:
         assert 0
 
     # Testing no num_nodes (shouldn't fail)
-    task = exctr.submit(calc_type='sim', num_procs=2, ranks_per_node=2, app_args=args_for_sim)
+    task = exctr.submit(calc_type='sim', num_procs=2, procs_per_node=2, app_args=args_for_sim)
     assert 1
-    task = polling_loop(exctr, task, delay=0.02)
+    task = polling_loop(exctr, task, delay=0.05)
     assert task.finished, "task.finished should be True. Returned " + str(task.finished)
     assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
 
-    # Testing no ranks_per_node (shouldn't fail)
+    # Testing no procs_per_node (shouldn't fail)
     task = exctr.submit(calc_type='sim', num_nodes=1, num_procs=2, app_args=args_for_sim)
     assert 1
-    task = polling_loop(exctr, task, delay=0.02)
+    task = polling_loop(exctr, task, delay=0.05)
     assert task.finished, "task.finished should be True. Returned " + str(task.finished)
     assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
 
@@ -468,7 +479,7 @@ def test_launch_as_gen():
     else:
         assert 0
 
-    exctr.register_calc(full_path=sim_app, calc_type='gen')
+    exctr.register_app(full_path=sim_app, calc_type='gen')
     task = exctr.submit(calc_type='gen', num_procs=cores, app_args=args_for_sim)
     task = polling_loop(exctr, task)
     assert task.finished, "task.finished should be True. Returned " + str(task.finished)
@@ -483,18 +494,6 @@ def test_launch_as_gen():
         assert 0
 
 
-def test_launch_default_reg():
-    print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
-    setup_executor_noreg()
-    exctr = Executor.executor
-    cores = NCORES
-    args_for_sim = 'sleep 0.1'
-    task = exctr.submit(calc_type='sim', num_procs=cores, app_args=args_for_sim)
-    task = polling_loop(exctr, task)
-    assert task.finished, "task.finished should be True. Returned " + str(task.finished)
-    assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
-
-
 def test_launch_no_app():
     print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
     setup_executor_noapp()
@@ -597,7 +596,7 @@ def test_retries_run_fail():
     exctr.retry_delay_incr = 0.05
     cores = NCORES
     args_for_sim = 'sleep 0 Fail'
-    task = exctr.submit(calc_type='sim', num_procs=cores, app_args=args_for_sim, wait_on_run=True)
+    task = exctr.submit(calc_type='sim', num_procs=cores, app_args=args_for_sim, wait_on_start=True)
     assert task.state == 'FAILED', "task.state should be FAILED. Returned " + str(task.state)
     assert task.run_attempts == 5, "task.run_attempts should be 5. Returned " + str(task.run_attempts)
 
@@ -606,8 +605,8 @@ def test_register_apps():
     print("\nTest: {}\n".format(sys._getframe().f_code.co_name))
     setup_executor()  # This registers an app my_simtask.x (default sim)
     exctr = Executor.executor
-    exctr.register_calc(full_path='/path/to/fake_app1.x', app_name='fake_app1')
-    exctr.register_calc(full_path='/path/to/fake_app2.py', app_name='fake_app2')
+    exctr.register_app(full_path='/path/to/fake_app1.x', app_name='fake_app1')
+    exctr.register_app(full_path='/path/to/fake_app2.py', app_name='fake_app2')
 
     # Check selected attributes
     app = exctr.get_app('my_simtask.x')
@@ -636,14 +635,50 @@ def test_register_apps():
         # assert e.args[1] == "Registered applications: ['my_simtask.x', 'fake_app1', 'fake_app2']"
 
 
+def test_serial_exes():
+    setup_serial_executor()
+    exctr = Executor.executor
+    args_for_sim = 'sleep 0.1'
+    task = exctr.submit(calc_type='sim', app_args=args_for_sim, wait_on_start=True)
+    task.wait()
+    assert task.finished, "task.finished should be True. Returned " + str(task.finished)
+    assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
+
+
+def test_serial_startup_times():
+    setup_executor_startups()
+    exctr = Executor.executor
+
+    t1 = time.time()
+    task = exctr.submit(app_name='c_startup')
+    task.wait()
+    stime = float(task.read_stdout())
+    startup_time = stime - t1
+    print('start up time for c program', startup_time)
+    assert task.finished, "task.finished should be True. Returned " + str(task.finished)
+    assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
+    assert 0 < startup_time < 1, "Start up time for C program took " + str(startup_time)
+
+    t1 = time.time()
+    task = exctr.submit(app_name='py_startup')
+    task.wait()
+    stime = float(task.read_stdout())
+    startup_time = stime - t1
+    print('start up time for python program', startup_time)
+    assert task.finished, "task.finished should be True. Returned " + str(task.finished)
+    assert task.state == 'FINISHED', "task.state should be FINISHED. Returned " + str(task.state)
+    assert 0 < startup_time < 1, "Start up time for python program took " + str(startup_time)
+
+
 if __name__ == "__main__":
-    # setup_module(__file__)
+    setup_module(__file__)
     test_launch_and_poll()
     test_launch_and_wait()
     test_launch_and_wait_timeout()
-    test_launch_wait_on_run()
+    test_launch_wait_on_start()
     test_kill_on_file()
     test_kill_on_timeout()
+    test_kill_on_timeout_polling_loop_method()
     test_launch_and_poll_multitasks()
     test_get_task()
     test_procs_and_machinefile_logic()
@@ -651,7 +686,6 @@ if __name__ == "__main__":
     test_finish_and_kill()
     test_launch_and_kill()
     test_launch_as_gen()
-    test_launch_default_reg()
     test_launch_no_app()
     test_kill_task_with_no_submit()
     test_poll_task_with_no_submit()
@@ -659,4 +693,6 @@ if __name__ == "__main__":
     test_retries_launch_fail()
     test_retries_run_fail()
     test_register_apps()
-    # teardown_module(__file__)
+    test_serial_exes()
+    test_serial_startup_times()
+    teardown_module(__file__)
